@@ -189,18 +189,20 @@ class line:
     def __init__(self,p1 : vect or tuple or list = None,p2 : vect or list = None,**kwargs) :
         
         # NEW : now line object can be constructed using list or tuples also.
-
-        if type(p1) == type(p2) == vect :
-            a = p1
-            b = p2
-        elif type(p1) == type(p2) == tuple or type(p1) == type(p2) == list :
-            a = vect(p1[0],p1[1],p1[2])
-            b = vect(p2[0],p2[1],p2[2])
-        elif p1 == p2 == None :
-            a = None
-            b = None
-        # ---
+        a = None
+        b = None
         
+        if p1 != None and p2 != None :
+            if type(p1) == vect :
+                a = p1
+            else :
+                a = vect(p1)
+            # ---
+            if type(p2) == vect :
+                b = p2
+            else :
+                b = vect(p2)
+
         # Point-direction form :
         
         if a != None and b != None :
@@ -257,6 +259,9 @@ class line:
     def __str__(self):
         return "line : point = " + str(self.point) + " , direction = " + str(self.direction) 
 
+    def __repr__(self):
+        return "line : point = " + str(self.point) + " , direction = " + str(self.direction) + '\n' + '[line object from vectogebra.geometry module]'
+
 
     # function to check if the line includes (or passes through) a point :
     def includes(self,r : vect) :
@@ -278,10 +283,9 @@ class line:
         # as cross product of propotional vectors (either parallel or anti-parallel) is 0, 
         # we have :-
 
-        if (r-self.point)^self.direction == 0 :
-            return True
-        else :
-            return False
+        return abs((r-self.point)^self.direction) == 0 
+        
+        # bug fixed 
 
 
     # function to check if the given two lines are parallel
@@ -298,15 +302,22 @@ class line:
         Returns : True if the lines are parallel, False otherwise.
 
         """
-        if (self.direction^other.direction).magnitude == 0 :
-            return True
+        if type(other) ==type(self):    
+            if abs(self.direction^other.direction) == 0 :
+                return True
+            else :
+                return False
+        elif type(other) == plane :
+            return (self.direction)*(other.normal) == 0 
         else :
-            return False
+            raise Exception("vectogebra.geometry.line.parallel() only accepts line or plane as arguments.")
+
     
     #function to find  shorest distance between two skew lines, parallel lines, or between a line and a point. 
     def distance(self,other) :
         """
-        Function to find shortest distance between two lines(skew or parallel), or between a line and a point.
+        Function to find shortest distance between two lines(skew or parallel), or between a line and a point,
+        or between a line and a plane.
 
         ---
 
@@ -357,83 +368,114 @@ class line:
             a = self.point
             b = self.direction
             p = other
-            return (p-a)^(b/b.magnitude)
+            return abs((p-a)^(b/b.magnitude))
+
+        elif type(other) == list or type(other) == tuple or type(other) == dict or type(other) == str :
+            a = self.point
+            b = self.direction
+            p = vect(other)
+            return abs((p-a)^(b/b.magnitude))
+
+        else :
+            raise Exception("vectogebra.geometry.line.distance() only accepts line or plane or point as arguments.")
 
 
     
     # function to check if two lines intersect or not :
     def intersects(self,other) :
         """
-        Function to check if two lines intersect or not. OR a line and a point intersects or not.
+        Function to check if two lines intersect or not. OR a line and a point intersects or not,
+        OR a line and a plane intersects or not.
 
         ---
 
-        argument : another `line` object. OR a point (`vector`)
+        argument : another `line` object, OR a point (`vector` or list or tuple or str : 'x y z'), OR a plane
 
         ---
 
-        Returns : True if the lines intersect, False otherwise.
+        Returns : True if the lines intersect with other object, False otherwise.
 
         """
         # The previous logic was wrong, as it ignored skew lines.
-        if self.distance(other) == 0 :
-            return True
-        else :
-            return False
+        return self.distance(other) == 0 
+        
 
     def intersection(self,other) :
         """
-            Function to find the position vector of the intersection point (if exists), of two lines.
+            Function to find the position vector of the intersection point (if exists), of two lines,
+            or between a line and a plane.
 
             ---
 
-            The argument should also be a line object.
+            The argument should also be a line object or a plane object.
         """
-        # extracting vectors from the lines :
-        # ---
-        P1 = self.point
-        D1 = self.direction
-        # ---
-        P2 = other.point
-        D2 = other.direction
-        # ---
-        # --- --- --- 
+        if type(other) == type(self) :    
+            # extracting vectors from the lines :
+            # ---
+            P1 = self.point
+            D1 = self.direction
+            # ---
+            P2 = other.point
+            D2 = other.direction
+            # ---
+            # --- --- --- 
 
-        # extracting the components of vectors defined above.
-        # ---
-        p = P1.x
-        q = P1.y
-        r = P1.z
-        # ---
-        s = D1.x
-        t = D1.y
-        u = D1.z
-        # ---
-        # ---
-        a = P2.x
-        b = P2.y
-        c = P2.z    
-        # ---
-        f = D2.x
-        g = D2.y
-        h = D2.z
-        # ---
-        # --- --- ---
-        if self.intersects(other) : 
-            if (f*t) != (s*g) :
-                phi = ((t*(p-a))-(s*(q-b)))/((f*t)-(s*g))
-            elif (u*f) != (s*h) :
-                phi = ((u*(p-a))-(s*(r-c)))/((u*f)-(s*h))
-            elif (u*g) != (t*h) :
-                phi = ((u*(q-b))-(t*(r-c)))/((u*g)-(t*h))
+            # extracting the components of vectors defined above.
+            # ---
+            p = P1.x
+            q = P1.y
+            r = P1.z
+            # ---
+            s = D1.x
+            t = D1.y
+            u = D1.z
+            # ---
+            # ---
+            a = P2.x
+            b = P2.y
+            c = P2.z    
+            # ---
+            f = D2.x
+            g = D2.y
+            h = D2.z
+            # ---
+            # --- --- ---
+            if self.intersects(other) : 
+                if (f*t) != (s*g) :
+                    phi = ((t*(p-a))-(s*(q-b)))/((f*t)-(s*g))
+                elif (u*f) != (s*h) :
+                    phi = ((u*(p-a))-(s*(r-c)))/((u*f)-(s*h))
+                elif (u*g) != (t*h) :
+                    phi = ((u*(q-b))-(t*(r-c)))/((u*g)-(t*h))
+                else :
+                    raise ValueError("Error in function line.intersects().")
+
+                INTERSECTION = P2 + (phi*D2)
+                return INTERSECTION
+
+            else:
+                return None
+        #---
+
+        elif type(other) == plane :
+            return other.intersection(self) #alredy definde in plane class.
+
+        elif type(other) == vect:
+            if self.contains(other) :
+                return other
             else :
-                raise ValueError("Error in function line.intersects().")
+                return None
+        
+        elif type(other) == list or type(other) == tuple or type(other) == dict or type(other) == str :
+            if self.contains(vect(other)) :
+                return vect(other)
+            else :
+                return None
 
-            INTERSECTION = P2 + (phi*D2)
-            return INTERSECTION
+        else : 
+            raise Exception("vectogebra.geometry.line.intersection() only accepts line or plane as arguments.")
+        
 
-        else:
-            return None
 
     # dunder methods
     
@@ -451,17 +493,20 @@ class line:
         Returns : True if the lines are equal, False otherwise.
 
         """
-        L1 = self
-        L2 = other
+        if type(other) == type(self) :   
+            L1 = self
+            L2 = other
 
-        a1 = self.point
-        b1 = self.direction
+            a1 = self.point
+            b1 = self.direction
 
-        a2 = other.point
-        b2 = other.direction
+            a2 = other.point
+            b2 = other.direction
 
-        if L1.includes(a2) and L2.includes(a1) and L1.parallel(L2) :
-            return True
+            if L1.includes(a2) and L2.includes(a1) and L1.parallel(L2) :
+                return True
+            else :
+                return False
         else :
             return False
 
@@ -487,6 +532,13 @@ class plane(object) :
     ```
     plane((x1,y1,z1),(x2,y2,z2),(x3,y3,z3))
     ``` 
+    ### 3. by equation of plane :
+    ```
+        plane('ax + by + cz = d')
+        plane('ax - by - cz = -d')
+        plane('ax -by -cz = d')
+        plane('-ax + by + cz = -d')
+    ```
     - vector `vectogebra.vector(x,y,z)` object or lists of components `[x,y,z]` can also be used instead of tuples.
     ---
 
@@ -536,21 +588,32 @@ class plane(object) :
         #---
 
         if p1 != None and p2 != None and p3 != None :
+            
             if type(p1) == vect :
                 v1 = p1
-            elif type(p1) == list or type(p1) == tuple :
+            elif type(p1) == list or type(p1) == tuple or type(p1) == str or type(p1) == dict:
                 v1 = vect(p1)
+            else :
+                raise TypeError("Error in constructor of plane")
+            #---    
             if type(p2) == vect:
                 v2 = p2
-            elif type(p2) == list or type(p2) == tuple :
+            elif type(p2) == list or type(p2) == tuple or type(p2) == str or type(p2) == dict:
                 v2 = vect(p2)
+            else :
+                raise TypeError("Error in constructor of plane")
+            #---
             if type(p3) == vect:
                 v3 = p3
-            elif type(p3) == list or type(p3) == tuple :
+            elif type(p3) == list or type(p3) == tuple or type(p3) == str or type(p3) == dict:
                 v3 = vect(p3)
-            
+            else :
+                raise TypeError("Error in constructor of plane")
+            #---
             self.point = v1
-            self.normal = (v2-v1)^(v3-v1)
+            self.normal = (v2-v1)^(v3-v1) 
+        else : 
+            pass
 
                 
 
@@ -564,6 +627,12 @@ class plane(object) :
             pass
         
         #---
+
+        # plane from its equation :
+        if p1 != None and p2 == None and p3 == None and type(p1) == str :
+            self.normal = PlaneEquationParser(p1)['normal']
+            self.point = PlaneEquationParser(p1)['point']
+        
         
         # shorthands :
         self.p = self.point
@@ -602,7 +671,7 @@ class plane(object) :
     #------------+----------------------+--------------# 
 
 
-    def includes(self,arg:vect or tuple or list or line or str):
+    def includes(self,arg:vect or tuple or list or line or str or dict):
         """
         Function to check :
         1. if a point is on the plane, or
@@ -635,20 +704,19 @@ class plane(object) :
             line0 = arg
 
         #---
-        if point !=None and abs(self.normal^(point-self.point)) == 0 :
+        if point !=None and abs(self.normal*(point-self.point)) == 0 :
             return True
         # a line is said to be contained in a plane if : 
         # 1. the direction of it is perpendicular to the normal vector of the plane.
         # 2. the line and the plane have atleast one point in common.
         elif line0 != None and  line0.direction*self.normal == 0 :
-            # the following is the condition required while solvinf equation of a line and a plane.
-            # the quantities which are crosseed must be proportional to each other. that's why the cross product is ZERO.
-            return abs((line0.point*(self.normal - self.point))^(line0.direction*(self.normal - self.point))) == 0
+            # the line should pe parallel to the plane. and its distance from the plane should be 0.
+            return self.distance(line0) == 0
         else :
             return False
     
     
-    def contains(self,arg:vect or tuple or list or line) :
+    def contains(self,arg:vect or tuple or list or line or str or dict) :
         return self.includes(arg)
 
     def distance(self, other : vect or line or tuple or list or dict or str ) -> int or float :
@@ -694,7 +762,7 @@ class plane(object) :
             n2 = other.n
             p1 = self.p
             p2 = other.p
-            if vut.proportional(n1,n2):
+            if abs(n1^n2) == 0 :
                 return abs((p2-p1)*(n1/n1.magnitude))
             else :
                 return 0
@@ -754,28 +822,31 @@ class plane(object) :
 
                 # finally the line is :
                 return line(point = c,direction = b)
-
-            elif type(other) == line :
-
-                p = self.point
-                n = self.normal
-
-                #---
-                a = other.point
-                b = other.direction
-                #---
-                
-                lmbd = ((p-a)*n)/(b*n)
-                #---
-                c = a + lmbd*b
-
-                return c
-                
             else :
                 return None
+        
+        #---                                        major bug fixed :-
+
+        elif type(other) == line :
+
+            p = self.point
+            n = self.normal
+
+            #---
+            a = other.point
+            b = other.direction
+            #---
+            if b*n != 0 :
+                lmbd = ((p-a)*n)/(b*n)
+                c = a + lmbd*b
+                return c
+            else :
+                return None
+
         #---
+
         else :
-            return None
+            raise TypeError("vectogebra.geometry.plane.intersection : argument must be a plane or a line.")
 
     def parallel(self,other : line or any ) -> bool :
         """
@@ -806,4 +877,96 @@ class plane(object) :
             return n*b == 0
         else :
             raise TypeError("vectogebra -> geometry.py -> plane -> parallel : argument must be a line or a plane.")
-    
\ No newline at end of file
+    
+
+# +----------------------------+
+# |    PLANE EQUATION PARSER   |
+# +----------------------------+
+def PlaneEquationParser(eqn:str) :
+
+    """
+        Function to parse the plane equation.
+
+        argument : 
+
+        the plane equation in string format.
+        The following equations are velid : 
+        - 'ax + by + cz = d'
+        - 'ax - by - cz = -d'
+        - '-ax -by -cz = d'
+        - '-ax + -by + -cz = -d
+        - 'ax - -by - -cz = d'
+        - '-ax - by - cz = -d'
+        Returns : a dict containing normal and a point on the plane.
+
+    """        
+    
+    
+    list_eqn = eqn.split(' ')
+    dict_eqn = {'x_coeff':0,'y_coeff':0,'z_coeff':0,'constant':0}
+
+    for i in list_eqn :
+        if 'x' in i :
+            if len(i) != 1 :    
+                if list_eqn[list_eqn.index(i)-1] == '+' :    
+                    dict_eqn['x_coeff'] = float(i.replace('x',''))
+                elif list_eqn[list_eqn.index(i)-1] == '-' :
+                    dict_eqn['x_coeff'] = -float(i.replace('x',''))
+                else :
+                    dict_eqn['x_coeff'] = float(i.replace('x',''))
+            else :
+                dict_eqn['x_coeff'] = 1
+        if 'y' in i :
+            if len(i) != 1 :   
+                if list_eqn[list_eqn.index(i)-1] == '+' :    
+                    dict_eqn['y_coeff'] = float(i.replace('y',''))
+                elif list_eqn[list_eqn.index(i)-1] == '-' :
+                    dict_eqn['y_coeff'] = -float(i.replace('y',''))
+                else :
+                    dict_eqn['y_coeff'] = float(i.replace('y',''))
+            else :
+                dict_eqn['y_coeff'] = 1
+        if 'z' in i :
+            if len(i) != 1 :    
+                if list_eqn[list_eqn.index(i)-1] == '+' :    
+                    dict_eqn['z_coeff'] = float(i.replace('z',''))
+                elif list_eqn[list_eqn.index(i)-1] == '-' :
+                    dict_eqn['z_coeff'] = -float(i.replace('z',''))
+                else :
+                    dict_eqn['z_coeff'] = float(i.replace('z',''))
+            else :
+                dict_eqn['z_coeff'] = 1
+
+        if '=' in i :
+            dict_eqn['constant'] = float(list_eqn[list_eqn.index(i)+1])
+    
+    nx = dict_eqn['x_coeff']
+    ny = dict_eqn['y_coeff']
+    nz = dict_eqn['z_coeff']
+    d = dict_eqn['constant']
+    
+    normal = vect(nx, ny, nz)
+    # for point point :
+    if nx != 0 and ny != 0 and nz != 0 : 
+        px = 1
+        py = 1
+        pz = (d-nx-ny)/nz
+        point = vect(px,py,pz)
+    elif nx != 0 and ny != 0 and nz ==0:
+        px = 1
+        py = (d-nx)/ny 
+        pz = 0
+        point = vect(px,py,pz)
+    elif nx != 0 and ny == 0 and nz != 0:
+        px = 1
+        py = 0
+        pz = (d-nx)/nz
+        point = vect(px,py,pz)
+    elif nx == 0 and ny != 0 and nz != 0:
+        px = 0
+        py = 1
+        pz = (d-ny)/nz
+        point = vect(px,py,pz)
+
+    return {'normal':normal, 'point':point}
+
